#!/usr/local/bin/perl
# 上記パスは、システムにあわせて変更してください

# 農薬検査所DB用携帯農薬検索システム(メイン CGI)
# Mobile Agricultural Chemical Searcher for ACIS
# Version 2.5.4.2 Copyright(C) 2004-2007 Hidemi Oya

# モジュール置き場にパスを通す
BEGIN {
  $lib = '../lib'; # !!!重要!!! モジュールを置いたフォルダーを指定
  unshift(@INC, $lib);
}

### ここから使用不可グローバル変数一覧 ###
## プログラム制御変数(必要に応じてユーザが値を設定)
$ma = './';        # このスクリプトのファイル名(index.cgi の場合は './' を指定)
$password= '';     # 利用者制限したい場合はパスワードを設定
$usedic = 1;       # 自動かな漢字変換使用フラッグ(1 で使用)
$smax = 30;        # 最大セッション数(0 の場合は無制限)
$sexp = 10;        # セッションの有効期限(分単位、5～15 を推奨)
$sdir = '../.smd'; # セッションデータ保存ディレクトリ
$hdir = './help';     # ヘルプルートディレクトリ
$tdir = './template'; # テンプレートルートディレクトリ
@help = ('', '', '', '/jsky', '/jsky', '', '', '','');      # ヘルプディレクトリ配列
@temp = ('', '', '', '/jsky', '/jsky', '/doti', '', '', ''); # テンプレートディレクトリ配列
  # ディレクトリ配列の要素番号
  #   0: PC, 1: EZweb WML, 2: EZweb XHTML, 3: SoftBank HTML, 
  #   4: SoftBank XHTML, 5: dot-i, 6: AIR-EDGE, 7: i/L-mode, 8: PDA
## CGI 内全域使用変数(ユーザが値を設定してはいけない)
$target = 'http://www.acis.famic.go.jp/search'; # ターゲット URI
$uri = $ma;  # URI 保存変数
$pi = '';    # CGI プログラム ID 保存
$si = '';    # セッション ID 保存
$sm = undef; # セッションマネージャオブジェクト保存
$jc = undef; # Unicode::Japanese オブジェクト保存
$is_svr = 0; # サーバデータフラッグ(サーバデータなら 1)
$body = '';  # 農薬DBからの取得内容
$_ = '';     # 携帯端末出力内容(システム変数)
### ここまで使用不可変数一覧 ###

# ドキュメント正規化(後処理横着化＆パケットサイズ削減のために)
sub normalize {
  s/^\s+//gm;       # 行頭空白削除
  s/\s+$//gm;       # 行末空白削除
  s/\n+//gs;        # 空行削除
  s/\&nbsp;?//ig;   # 空白文字を削除(;? は農薬検査所検索システムのバグ対策)
  s/\s+/ /g;        # 複数空白を１つのスペースに
  s/ += +/=/g;      # '=' の前後のスペースを削除
  s/(<|<!--) /\1/g; # タグ／コメント開始直後のスペースを削除
  s/ (>|-->)/\1/g;  # タグ／コメント終了直前のスペースを削除
  s%<([^a-z!/\?])%&lt;\1%ig;  # タグ以外の '<' を '&lt;' に変換
  s/([^\w'"\-\?])>/\1&gt;/ig; # タグ以外の '>' を '&gt;' に変換
  s/="([^";\s]+)"/=\1/g; # クォート内にセミコロン／空白を含まないクォートを削除
  s/='([^';\s]+)'/=\1/g; # クォート内にセミコロン／空白を含まないクォートを削除
}

# 受信データ整形
sub regularize {
  my $tag = shift;
  $body =~ s/^.*?($tag)/\1/i;                # 先頭から最初の $tag までを削除
  $body =~ s/<table class=footer.*$//i;      # <table class=footer 以降を削除
  $body =~ s%(/table>).+?(<table|$)%\1\2%ig; # </table> の後ろにある不要タグ削除
  $body =~ s/ class=[^ >]*//ig;              # class 属性を削除
  $body =~ s/ width=[^ >]*//ig;              # width 属性を削除
  $body =~ s/<p[^>]*>//ig;                   # <P> を削除
  $body =~ s|</p>(</t[dh]>)|\1|ig;           # </TD>|</TH> 直前の </P> を削除
  $body =~ s|</p>|<br>|ig;                   # </P> を <BR> に変換
}

# メニューデータ整形
sub stripmenu {
  $body =~ s/^.*?(?!\+Menu)//i;
  $body =~ s/\+'.*$//;
  $body =~ s/\+Menu\([^']*'(empty)?folder',[^\)]*\)//ig;
  $body ='' if ($body !~ /\+Menu/i);
  $sm->param('menu', $body);
  $sm->flush;
}

# ユーザエージェント判定
sub agent {
  my $pat = '(^UP\.Browser)|(^KDDI)|'.       # EzWeb: WML = 1, XHTML = 2
            '(^J-PHONE)|(^Vodafone|^MOT-|^SoftBank)|'. # SoftBank: HTML = 3, XHTML = 4
            '(^ASTEL)|(^PDXGW)|'.            # dot-i = 5, Air-EDGE = 6
            '(^DoCoMo|^L-mode)|'.            # i/L-mode = 7
            '(Palm|Windows CE|'.             # PDA = 8
            '^sharp pda|CASSIOPEIA)';
  my $ret = 0;                               # 上記以外 = 0
  # HTTP_USER_AGENT が未設定の端末は SoftBank のエミュレータとみなす
  $ENV{'HTTP_USER_AGENT'} = 'SoftBank Emulator' if (!$ENV{'HTTP_USER_AGENT'});
  @matches = ($ENV{'HTTP_USER_AGENT'} =~ /$pat/);
  if (@matches) {
    for($ret = 0; $ret < @matches; $ret++) {
      last if ($matches[$ret]);
    }
    $ret++;
  }
  return($ret);
}

# Google 用セットアップ
sub google {
  return('') if (!@_); # キーワードがなければ終了
  my $uri = 'http://www.google.co.jp/';
  my @ua = ('search', 'wml', 'xhtml', 'jsky', 'xhtml', 'imode', 'imode', 'imode', 'pda');
  # キーワードの URI エンコード
  $" = ' '; # 配列展開デリミタをスペースに設定
  my $kw = "@_"; # キーワードを全て展開
  $kw =~ s/(\W)/'%'.unpack('H2', $1)/eg; # エンコード
  $kw =~ s/%20/+/; # '%20' を '+' に変換
  # Google 検索へのリンクを返す
  return("<A href=$uri$ua[&agent]?q=$kw>$_[$#_]</A>");
}

# データソース表示
sub source {
  return if (&agent); # 携帯端末なら表示しない
  # データソース判定
  my $src = 'Session';
  $src = 'Server' if ($is_svr);
  s|(</title>)| ($src)\1|i;  # タイトルにデータソースを追加
}

# 作物名・病害虫名等の取得
sub getname {
  my $code;
  my $list;
  if ($_[0] eq 'crop') {
    $code = $sm->param('cv');
    $list = $sm->param('k101');
  } else {
    $code = $sm->param('pv');
    $list = $sm->param('k102');
  }
  return($1) if ($list =~ /,'$code','([^']+)/);
  return($_[1]);
}

# サーバサイド CGI へ渡すパラメータの設定
sub setparam {
  return if (($pi ne 'g304') && ($pi ne 'g103'));
  my @param = ('currentPage', 'syh_mk_name',
               'skmt[0].value', 'skmt[0].label',
               'byg[0].value', 'byg[0].label',
               'nyk_name', 'nyk_syu_name', 'zai.value', 'zai.label',
               'trk_date.year', 'trk_date.month', 'trk_date.day',
               'yuko_date.year', 'yuko_date.month', 'yuko_date.day');
  my $g303 = $sm->param('g303');
  foreach $key (@param) {
    my $r = $key;
    $r =~ s/([\[\]\.])/\\\1/g;
    $g303 =~ /name=$r value=([^>]*)/i;
    my $val = $1;
    $val = '' if (($val eq "''") || ($val eq '""'));
    $PARAMS{$key} = $val;
  }
  $PARAMS{'clickTrk_no'} = $rn;
}

# エラー処理
sub errproc {
  my %arg = (file => 'errsys.htm', subj => 'システムエラー', msg => '', src => 0, back => '', @_,);
  # エラーテンプレート読み込み
  open(HTM, "$tdir/$arg{'file'}");
  my @fs = stat(HTM);
  read(HTM, $_, $fs[7]);
  close(HTM);
  &normalize;
  # エラー画面設定
  s/\$title/$arg{'subj'}/i;
  s/\$message/$arg{'msg'}/i;
  s/\$back/$uri&pi=$arg{'back'}/i; # バック URL の設定
  &source if ($si && $arg{'src'}); # タイトルにデータソースを追加
  exit 0; # END 処理にジャンプ
}

# ヘルプエラー
sub helperr {
  my $hi = shift;
  &errproc(file => 'erracc.htm', subj => 'ヘルプエラー', msg => "$hi のヘルプファイルが見つかりません", back => $hi);
}

# データエラー
sub dataerr {
  (my $message, my $subject) = @_;
  $subject = '受信データエラー' if (!$subject);
  if ($pi eq 'k101') {
    my $ecn = $sm->param('ecn');
    $message .= "<BR><BR>入: $ecn<BR>検: $cn";
  }
  if ($pi eq 'k102') {
    my $epn = $sm->param('epn');
    $message .= "<BR><BR>入: $epn<BR>検: $pn";
  }
  # エラー復帰時プログラム ID
  *prev = {sc => 'sc', k101 => 'sc', sp => 'sp', k102 => 'sp', sa => 'sa', g303 => 'sa', g103 => 'g303', g304 => 'g103'};
  &errproc(file => 'erracc.htm', subj => $subject, msg => $message, src => 1, back => $prev{$pi});
}

# 接続エラー
sub cncterr {
  my $errcode = shift;
  my $message = '農薬検査所に接続できません<BR>'.
                '時間をおいて再アクセスしてください';
  &errproc(subj => "接続エラー $errcode", msg => $message);
}

# セッションエラー
sub seserr {
  $si = '';
  my ($message, $subject) = @_;
  $subject ||= 'セッションエラー';
  &errproc(subj => $subject, msg => $message);
}

# システムエラー
sub syserr {
  my $message = '';
  foreach $msg (@_) {
    $message .= "$msg<BR>";
  }
  &errproc(msg => $message);
}

# テンプレートファイルロード
sub readtemp {
  my %errmsg = (sc => '作物名等設定', k101 => '作物名等選択',
                sp => '病害虫名等設定', k102 => '病害虫名等選択',
                sa => '農薬名等設定', g303 => '農薬選択',
                g103 => '適用表表示', g304 => '農薬登録票表示',
                pw => 'パスワード入力');
  my $file = shift;
  $file = $pi if (!$file);
  open(HTM, "$tdir/$file.htm") or &syserr("$errmsg{$file}画面が見つかりません");
  my @fs = stat(HTM);
  read(HTM, $_, $fs[7]);
  close(HTM);
  &normalize;
}

# ID エンコード(6bit)
sub encid {
  my @c = ('0'..'9', 'A'..'Z', 'a'..'z', '-', '_');
  my ($num, $dig) = @_;
  my $res = '';
  while ($dig) {
    $res = $c[$num & 0x3F].$res;
    $num >>= 6;
    $dig--;
  }
  return($res);
}

### ここから本処理開始 ###
# 使用モジュールロード
use Unicode::Japanese;
$jc = Unicode::Japanese::PurePerl->new;
use HTTP::Lite;
use HTTP::Date;
use CGI::Session;
my $sesver = $CGI::Session::VERSION;
&syserr('CGI::Session version '.$sesver, '4.00 以上が必要です') if (index($sesver, '4.') != 0);
use CGI::Session::Inspect;

# ヘルプ／テンプレートディレクトリ初期化
$hdir .= $help[&agent];
$tdir .= $temp[&agent];

# クライアントサイド CGI から渡されたパラメータの一括読み込み
my $query;
if ($ENV{'REQUEST_METHOD'} eq 'POST') {
  read(STDIN, $query, $ENV{'CONTENT_LENGTH'});
} else {
  $query = $ENV{'QUERY_STRING'};
}
$query =~ tr/+/ /; # '+' を半角スペースに変換
$query =~ s/%([0-9A-Fa-f][0-9A-Fa-f])/pack("H2", $1 )/eg;  # エスケープ文字を復元
# クライアントサイド CGI から渡されたパラメータの取得
foreach $pair (split(/&/, $query)) {
  ($key, $value) = split(/=/, $pair);
  $key = lc($key); # key の小文字化
  $value = lc($value) if (index('pi hi', $key) >= 0); # pi, hi は value を小文字化
  # パラメータ名が an|ak|cn|pn|up の場合
  if (index('cn pn an ak up', $key) >= 0) {
    # Vodafone 3G Shift_JIS 非対応機種のための文字コード変換
    my $a = $jc->set($value, 'utf8')->sjis;
    $value = $a if ($value eq $jc->set($a, 'sjis')->utf8);
    # ひらがな->カタカナ変換、SJIS->UTF8変換
    $value = $jc->set($value, 'sjis')->hira2kata->utf8;
    # 英半角大文字化、半角->全角変換
    $value = $jc->set(uc($value), 'utf8')->h2z->utf8;
  }
  eval('$'.$key.' = $value'); # 変数に値を保存
}

# セッション管理
$dsn = 'id:static';
$hash = {Directory => $sdir};
# $si が設定されている場合正当性をチェック
if ($si) {
  # PC からのアクセスで本スクリプト以外から渡された ID は棄却
  &seserr('不正アクセスです:1', 'アクセスエラー')
    if ((index($ENV{'HTTP_REFERER'}, $ENV{'SERVER_NAME'}.$ENV{'REDIRECT_URL'}) != 7) && !&agent);
  # 不正な ID 文字列は棄却
  &seserr('不正アクセスです:2', 'アクセスエラー') if ($si !~ /^[\-_0-9A-Za-z]{5}$/);
  # セッションの監査
  my $s = CGI::Session->inspect($dsn, $si, $hash) or &seserr('セッションの取得に失敗しました');
  # 空 ID は棄却
  &seserr('セッションの有効期限が切れています') if ($s->is_empty);
  # REMOTE_HOST または USER_AGENT が異なる ID は棄却（REMOTE_ADDR が確実だが携帯電話では使えない）
  &seserr('不正アクセスです:3', 'アクセスエラー')
    if (($s->param('REMOTE_HOST') ne $ENV{'REMOTE_HOST'}) || ($s->param('USER_AGENT') ne $ENV{'HTTP_USER_AGENT'}));
  # セッション初期化
  $sm = CGI::Session->load($dsn, $si, $hash) or &seserr('セッションの初期化に失敗しました');
}
# $si が指定されていない場合新しいセッション ID を取得
if (!$si) {
  $pi = ''; # $si が指定されていない場合は $pi も初期化
  # 有効セッション数が最大セッション数以上の場合はエラー
  # valid_id メソッドはセッションを掃除して有効な ID リストを返す
  if ((CGI::Session->valid_id($dsn, $hash) >= $smax) && $smax) {
    &seserr("ただ今混雑しています。<BR>$sexp分後に再アクセスしてください。");
  }
  $si = &encid(time, 3).&encid($$, 2); # セッション ID 作成
  $sm = new CGI::Session($dsn, $si, $hash) or &seserr('セッションの作成に失敗しました');
  $sm->expire("\+${sexp}m"); # セッションの有効期限の設定
  $sm->param('USER_AGENT', $ENV{'HTTP_USER_AGENT'});
  $sm->param('REMOTE_HOST', $ENV{'REMOTE_HOST'});
  if ($usedic) { # 自動かな漢字変換使用設定
    $sm->param('cd', 1); # 作物名等
    $sm->param('pd', 1); # 病害虫名等
    $sm->param('ad', 1); # 使用目的等
  }
}
$sm->param('pw', $pw) if ($pw); # パスワードがあればセッションに保存
$sm->flush;
# URI の設定
$uri .= "\?si=$si";

# パスワードチェック
if ($password && ($password ne $sm->param('pw'))) {
  # 検索画面読み込み
  &readtemp('pw');
  exit 1; # END 処理にジャンプ
}

# $hi が定義されていればヘルプ画面表示
if (defined($hi)) {
  # ヘルプ画面読み込み
  open(HTM, "$hdir/$hi.htm") or &helperr('ヘルプ画面が見つかりません');
  @fs = stat(HTM);
  read(HTM, $_, $fs[7]);
  close(HTM);
  &normalize;
  exit 1; # END 処理にジャンプ
}

# $pi が未設定なら $pi を設定
if (!$pi) {
  if (defined($rn)) { # $rn が定義されていた場合 g103 へ
    $pi = 'g103';
  } elsif (defined($gt)) { # $gt が定義されていた場合 g303 へ
    $pi = 'g303';
  } elsif (defined($ag)) { # $ag が定義されていた場合 g303 へ
    $pi = 'g303';
  } elsif (defined($ak)) { # $ak が定義されていた場合 g303 へ
    $pi = 'g303';
  } elsif (defined($pv)) { # $pv が定義されていた場合 sa へ
    $pi = 'sa';
  } elsif (defined($pg)) { # $pg が定義されていた場合 k102 へ
    $pi = 'k102';
  } elsif (defined($pn)) { # $pn が定義されていた場合 k102 へ
    $pi = 'k102';
  } elsif (defined($cv)) { # $cv が定義されていた場合 sp へ
    $pi = 'sp';
  } elsif (defined($cg)) { # $cg が定義されていた場合 k101 へ
    $pi = 'k101';
  } elsif (defined($cn)) { # $cn が定義されていた場合 k101 へ
    $pi = 'k101';
  } else {
    $pi = 'sc';
  }
}

# サーバサイド CGI へ渡すパラメータの前処理等(JavaScript の代用ほか)実行
&syserr("snd$pi.pl の実行に失敗しました") if (!eval("require('snd$pi.pl')"));

# 検索項目入力画面表示
if (index('sc sp sa', $pi) >= 0) {
  exit 1; # END 処理にジャンプ
}

# 農薬登録情報取得
$body = $sm->param($pi); # セッションデータ読み込み
# データが存在しない場合は農薬検査所の検索 CGI にリクエスト
if (!$body) {
  my $http = new HTTP::Lite; # HTTP::Lite オブジェクトの作成
  $http->prepare_post(\%PARAMS); # POST パラメータの設定
  my $req = $http->request("$target/vtll$pi.do") or &syserr('サーバ接続方法が誤っています');
  &cncterr($req) if ($req ne '200');
  $_ = $http->body();
  &normalize;
  $body = $_;
  $is_svr = 1;
  # キャッシュ／セッションへのデータ保存
  $sm->param($pi, $body);
  $sm->flush;
}

# 取得 HTML の携帯端末向け整形処理
if (/\[(LL[A-Z][A-Z]E[0-9]+)\]:?([^<]+)/i) { # サーバサイドエラー処理
  &dataerr("$1<BR>$2", 'サーバサイドエラー');
} else { # 正常検索の処理
  # サーバサイド CGI 別の整形ルーチン呼び出し
  &syserr("rcv$pi.pl の実行に失敗しました") if (!eval("require('rcv$pi.pl')"));
}
&source; # タイトルにデータソースを追加

1;

# 終了ルーチン(必ずここを通って終了する)
END {
  if ($si) {
    s/\$cn/&getname('crop', '指定なし')/e; # 作物名等の表示
    s/\$pn/&getname('pest', '指定なし')/e; # 病害虫名等の表示
  }
  s/\$ma/$ma/g;    # テンプレート中の $ma を $ma の内容に書き換え
  s/\$si/$si/g;    # テンプレート中の $si を $si の内容に書き換え
  s/\$pi/$pi/g;    # テンプレート中の $pi を $pi の内容に書き換え
  s/\$uri/$uri/g;  # テンプレート中の $uri を $uri の内容に書き換え
  s/<!--.*?-->//g; # コメント削除
  $_ = $jc->set($_, 'utf8')->sjis; # 出力ファイルの一括文字コード変換
  s/(charset=)UTF-8/\1Shift_JIS/i; # 出力ファイルの charset 変換
  # 結果出力
  my $gtime = time2str;
  print "Cache-control: no-cache\n";
  print "Pragma: no-cache\n";
  print "Expires: $gtime\n";
  print "Date: $gtime\n";
  print "Content-type: text/html\n\n";
  print;
}

__END__

# 開発履歴
# 1.0.0 2004/12/22 ファーストリリース
# 1.1.0 2005/04/10 作物名・病害虫等名による検索機能追加
#                  ma.cgi を index.cgi に変更
#                  デバッグモードでは、HTTP::Lite を use しないように変更
# 1.2.0 2005/04/18 登録農薬一覧表示方法変更への対応
#                  作物等選択画面・病害虫等選択画面にパスワードが渡らないバグを修正
#                  setquery 関数に $SJIS{'cl'} パラメータの処理を追加
#                  その他ソース全般の見直し
# 1.2.1 2005/04/24 フォーム及び作物名一覧にパスワードが渡らないバグを修正
#                  これに対応し、setquery 関数を仕様変更
#                  スキップ・戻るメニューで不要なパラメータが渡るバグを修正
# 1.3.0 2005/05/13 受信パラメータ変数を $SJIS{'xx'} から $xx 形式に変更
#                  送信パラメータ変数を %UTF8 から %PARAMS に変更
#                  setparam 関数の仕様変更
#                  $cl を $cn に統合
#                  農薬一覧に戻る際に $at パラメータを渡していなかったバグを修正
# 1.4.0 2005/05/21 Vodafone 3G 端末に対応(google 関数、文字コード判定)
#                  google 関数の仕様変更(検索文字列を UTF8 で指定)
# 2.0.0 2005/05/26 サーバサイドセッションへの対応
#                  同時接続セッション数によるアクセス制限機能を追加
#                  ページごとのヘルプ表示機能追加
#                  キャリア別テンプレート読み込み機能の追加
#                  データソース表示機能追加(デバッグ用、固定端末のみ)
#                  google 関数の仕様変更(固定端末は固定端末用検索画面へ)
#                  setparam 関数を仕様変更
#                  エラー処理方法を変更
#                  内部処理の文字コードを UTF8 に変更(出力段階で一括して SJIS に変換)
#                  Jcode 対応部分の削除
# 2.0.1 2005/06/04 PDA に対応(agent, google 関数)
# 2.0.2 2005/09/27 適用データに '<' が含まれる場合、'<' 以降が表示されないバグ修正
#                  セッションデータクリア方法変更
# 2.5.0 2005/10/26 作物名／病害虫名等自動かな漢字変換対応
#                  農薬名から屋号を削除する関数(tmcut)を追加
#                  ブラウザのナビゲーションボタンで操作した場合に正常に動作しないことがあるバグを修正(-> デバッグモード削除)
#                  normalize でクォート内に空白を含む場合でもクォートを削除していたバグ修正
#                  受信データ整形処理(regularize, stripmenu 関数)追加(-> normalize 関数仕様変更)
#                  正規表現単純化と可能な場合は index 関数に変更
#                  作物等名／病害虫等名のパラメータ名変更
#                  google 関数に複数のパラメータを渡せるように仕様変更
#                  proxy や端末内キャッシュによる誤動作対策追加
#                  セッション管理方法変更
# 2.5.1 2005/10/27 携帯電話のキャリアによっては「不正アクセスです:1」でアクセスできないバグを修正
#                  Vodafone 3G 端末の一部機種で検索が正常にできない可能性があるバグを修正
# 2.5.2 2005/11/04 tmcut 関数を tm モジュールに移動
#                  ヘルプ／テンプレートディレクトリの指定方法変更
# 2.5.3 2005/11/09 作物名等／病害虫名等で検索エラーの場合に入力語／検索語を表示するよう変更
#       2006/03/30 使用目的検索に対応
# 2.5.4 2006/10/24 ソフトバンクモバイル対応
#       2007/04/01 農薬検査所組織改変に伴う URL 変更に対応
#       2007/11/25 Unicode::Japanese のバージョンによってスクリプトエラーになる不具合を修正
