# 農薬検査所DB用携帯農薬検索システム(サブ CGI)
# 作物名一覧(vtllk101.do)用出力変換
# Version 2.5.1 Copyright(C) 2005 Hidemi Oya

# 分類ランク取得関数
sub getclass {
  my $code = shift;
  return('') if (substr($code, 2) eq '00000000'); # 大分類
  return('.') if (substr($code, 4) eq '000000'); # 中分類
  return('..') if (substr($code, 6) eq '0000'); # 小分類
  return('...') if (substr($code, 8) eq '00'); # 代表作物名
  return('....');
}

&stripmenu; # 受信データ整形

# 出力画面整形
&readtemp; # テンプレート読み込み
# バック URI が不要なら削除
s/<!--begin_backuri-->.*?<!--end_backuri-->// if (!$cg);
# 入力語／検索語表示
my $ecn = $sm->param('ecn');
if ($ecn ne $cn) { # 入力語と検索語が異なる場合
  s/\$ecn/$ecn/i; # 入力語を表示
} else { # 入力語と検索語が同じ場合
  s/<!--begin_entered_word-->.*?<!--end_entered_word-->//; # 入力語表示部分を削除
}
s/\$cn/$cn/i; # 検索語を表示

# 作物名等の取得
my @classes = split(/\+Menu/i, $body);
my %crops = ();
foreach $class (@classes) {
  next if ($class !~ /,'([0-9]+)','([^']+)/);
  my $code = $1;
  my $name = $2;
  # 'を除く'を含む作物名等はスキップ
  next if (index($name, 'を除く') >= 0);
  my $gcode = substr($code, 0, 8);
  $crops{$gcode}{$code} = $name;
  $crops{$gcode}{$gcode.'00'} ||= "($name)"; # 代表作物名が存在しない場合の対策
}
&dataerr('該当する作物名等がありません', '検索エラー') if (!%crops);

# 作物名等一覧出力フォーマットの設定
my $blockid = '<!--begin_list-->(.+?)<!--end_list-->';
&syserr('作物名等一覧出力書式がありません') if (! /$blockid/i);
# 作物名等グループの出力フォーマット設定
&syserr('作物名等グループ出力書式がありません') if (! /<!--begin_group-->(.+?)<!--end_group-->/i);
my $frmgroup = $1;
# 作物名等個別データの出力フォーマット設定
&syserr('作物名等出力書式がありません') if (! /<!--begin_data-->(.+?)<!--end_data-->/i);
my $frmdata = $1;

# 作物名・分類一覧の作成
my $res = ''; # 結果保存変数の初期化
foreach $id (sort keys %crops) {
  next if ($cg && ($cg ne $id)); # $id と $cg が一致しなければスキップ
  my $count = keys(%{$crops{$id}});
  my $f_ind = $cg || ($count == 1); # 個別データ表示フラッグ
  foreach $code (sort keys %{$crops{$id}}) {
    my $name = $crops{$id}{$code}; # 作物名
    next if (index($name, '(') == 0); # 作物名の先頭が '(' の場合はスキップ
    my $class = &getclass($code);  # 分類ランク
    if ($f_ind) { # 個別データ表示
      eval('$res .= "'.$frmdata.'"'); # 作物名等個別データの整形
    } else { # グループ表示
      # 検索語と一致しない作物名はスキップ
      next if ($jc->set($name, 'utf8')->h2z->hira2kata->utf8 !~ /$cn/);
      my $gname = $crops{$id}{$id.'00'}; # グループ名
      my $gclass = &getclass($id.'00');  # グループランク
      eval('$res .= "'.$frmgroup.'"'); # 作物名等グループの整形
      last; # グループを表示したら終了
    }
  }
}
s/$blockid/$res/i; # 作物名等一覧の整形

1;
__END__

# 開発履歴
# 1.0.0 2005/04/07 ファーストリリース
# 1.1.0 2005/04/18 作物虫名等一覧表示方法変更
#                  該当病害虫名等が存在しない場合のエラー処理追加
# 1.1.1 2005/04/24 次画面にパスワードが渡らないバグを修正
# 1.2.0 2005/05/13 グループ展開時に作物名一覧に戻る機能を追加
#                  受信パラメータ変数の変更に対応
# 1.2.1 2005/05/21 「除く」を含む作物名等は非表示に変更
# 2.0.0 2005/05/26 エラー処理の変更に対応
#                  キャリア別テンプレート読み込み機能に対応
#                  データソース表示機能に対応
#                  内部処理の文字コードを UTF8 に変更
# 2.5.0 2005/10/26 データ整形処理追加
#                  正規表現単純化と可能な場合は index, substr 関数に変更
#                  検索語とマッチする作物名を代表作物名として表示するよう変更
#                  検索語表示機能追加(自動かな漢字変換機能への対応)
# 2.5.1 2005/11/17 「花き」のように代表作物名が存在しない作物分類で分類名が表示されないバグを修正
